﻿var Diagram = MindFusion.Diagramming.Diagram;
var Behavior = MindFusion.Diagramming.Behavior;
var AbstractionLayer = MindFusion.AbstractionLayer;
var Alignment = MindFusion.Diagramming.Alignment;
var AnchorPattern = MindFusion.Diagramming.AnchorPattern;
var Shape = MindFusion.Diagramming.SimpleShape;
var Font = MindFusion.Drawing.Font;
var Rect = MindFusion.Drawing.Rect;
var Text = MindFusion.Drawing.Text;
var Thickness = MindFusion.Drawing.Thickness;
var Event = MindFusion.Diagramming.Events;
var GlassEffect = MindFusion.Diagramming.GlassEffect;
var Style = MindFusion.Diagramming.Style;
var SimpleShape = MindFusion.Diagramming.SimpleShape;
var ImageAlign = MindFusion.Diagramming.ImageAlign;
var LinkShape = MindFusion.Diagramming.LinkShape;

var id = 0;

// creates a table node with the default settings
var OrgChartNode = function (parent, boss)
{
	AbstractionLayer.initializeBase(OrgChartNode, this, [parent]);

	this.childNodes = [];

	// set up table cells
	this.redimTable(3, 4);
	//this.getCell(1, 2).setColumnSpan(0);
	this.getCell(0, 0).setRowSpan(4);
	this.getCell(0, 0).setFont(
            new Font("TERAFONT-PARTH", 8, true /*bold*/, false /*italic*/));
	this.getCell(0, 0).setTextAlignment(Alignment.Center);
	//this.getCell(1, 1).setText("Title:");
    //this.getCell(1, 1).setFont(
    //new Font("Verdana", 3.5, true /*bold*/, false /*italic*/));
	//this.getCell(1, 2).setFont(
      //      new Font("Verdana", 3.5, true /*bold*/, false /*italic*/));
	//this.getCell(1, 0).setFont(
      //      new Font("Verdana", 3.5, true /*bold*/, false /*italic*/));
	//this.getCell(1, 3).setFont(
      //      new Font("Verdana", 3, false /*bold*/, false /*italic*/));
	this.configureCells();

	// customize appearance
	this.setCaptionHeight(10);
	this.setHandlesStyle(MindFusion.Diagramming.HandlesStyle.MoveOnly);
	this.setCellFrameStyle(MindFusion.Diagramming.CellFrameStyle.None);
	this.setBrush("purple");
	this.setShape(SimpleShape.RoundedRectangle);
	this.setFullName("Name");
	//this.setTitle("title");
	this.setImageLocation("http://www.pi-cube.com/wp-content/uploads/2015/04/team-placeholder.jpg");
	this.setId();
	this.setBoss(boss);
	this.setHierarchy();

	this.resize();
	this.setAnchorPattern(AnchorPattern.fromId("TopInBottomOut"));
}

OrgChartNode.prototype =
{
	// updates the existing elements
	updateCanvasElements: function (node)
	{
		this.setFields();
		AbstractionLayer.callBaseMethod(OrgChartNode, this, 'updateCanvasElements');
	},

    // gets the title of the employee
	getTitle: function ()
	{
		return this.title;
	},

    // sets the title of the employee
	setTitle: function (value)
	{
		if (this.title !== value)
		{
			//this.title = value;
			this.invalidate();
		}
	},

    // gets the full name of the employee
	getFullName: function ()
	{
		return this.fullName;
	},

    // sets the full name of the employee
	setFullName: function (value)
	{
		if (this.fullName !== value)
		{
			this.fullName = value;
			this.invalidate();
		}
	},

    // gets the url where the employee picture is located
	getImageLocation: function ()
	{
		return this.nodeImageLocation;
	},

    // sets the url where the employee picture is located
	setImageLocation: function (value)
	{
		if (this.nodeImageLocation != value)
		{   //places the image in cell (0,0)
			//var alignment = MindFusion.Diagramming.ImageAlign;
			//this.nodeImageLocation = value;
			//this.getCell(0,0).setImageLocation(value);
			//this.getCell(0,0).setImageAlign(alignment.Stretch);
			//this.invalidate();
		}
	},

    // gets the comments about the employee
	getComment: function()
	{
		return this.comments;
	},

    // gets the comments about the employee
	setComment: function(value)
	{
		if (this.comments !== value)
		{
			this.comments = value;
			this.invalidate();
		}
	},

    // assigns an id to the employee
	setId: function ()
	{
		this.id = id;
		id++;
	},

    // gets the id that was assigned to the employee
	getId: function()
	{
		return this.id;
	},

    // assigns the employee data to the table cells
	setFields: function()
	{
        // hide the caption and place the employee names on row 0 
		this.setCaptionHeight(0);
		//this.getCell(1,0).setText("Name:")
		this.getCell(0, 0).setText(this.fullName);
		this.getCell(0, 0).setTextAlignment(Alignment.Center);

        // the employee title is on the second row
		//this.getCell(2,1).setText(this.title);

        // render the employee id at the third row
		//
        //this.getCell(1,2).setText("ID: "+this.id);

        // place the image in cell (0, 0)
		this.getCell(0,0).setImagePadding(new Thickness(0,0,0,0));
		this.getCell(0,0).setImageLocation(this.ImageLocation);
		this.getCell(0, 0).setImageAlign(ImageAlign.Fit);

        // identifies the id of the employee's boss
		//if(this.boss != undefined){
			//if (this.boss.id != undefined)
				//this.getCell(2,0).setText("");
		//}

        // sets the comments for the employee
		//this.getCell(1,3).setText(this.comments);
		//this.getCell(1,3).text.padding = new Thickness(0,0,0,0);

        // rearrange the org hierarchy
		this.setHierarchy();
		this.setColor();
	},

    // resize the node to guarantee that all data would fit
	resize: function()
	{
		var oldHeight = this.bounds.height;
		var oldImageWidth = this.getColumn(0).width;

        //resize the node to fit the text
		this.resizeToFitText(true,true);

        //add or remove the necessary space to fit the image well
		var newHeight = this.bounds.height - oldHeight;
		this.getCell(0, 0).imagePadding.top += newHeight/2;
		this.getCell(0, 0).imagePadding.bottom += newHeight / 2;
		this.getColumn(0).width = oldImageWidth;
		this.bounds.width += oldImageWidth;

	},

    // disable edit on the cells that do not contain employee data
    configureCells: function()
	{
		for(var i = 0;i<3;i++)
		{
			for(var j = 0;j<3;j++){
				this.getCell(i,j).editable = false;
			}
		}
        
		// specify text padding
		for(var i=0;i<4;i++){
			this.getCell(0,i).text.padding.left = 1;
			this.getCell(0,i).text.padding.right = 1;
}

        // allow text edit on cells with employee data
		//this.getCell(0,0).editable = true;
		//this.getCell(1,3).editable = true;
		//this.getCell(2,0).editable = true;
		//this.getCell(2,1).editable = true;
	},

    // assigns the specified boss as the employee's boss
	setBoss: function(boss)
	{
		if (boss != this.boss)
		{
			this.boss = boss;
			this.setHierarchy();
			this.invalidate();
		}
	},

    // rebuilds the hierarchy
	setHierarchy: function ()
	{
        // the ceo has no boss
		if (this.boss == undefined)
		{
			this.hierarchy = 0;
		}
		else
		{
            // first level of executives under the boss
			if (this.boss.hierarchy == undefined)
			{
				this.hierarchy = 1;
			}
			else
			{
				// increase the depth of the hierarchy
				this.hierarchy = this.boss.hierarchy + 1;
				this.boss.addChild(this);
			}
		}
        // rearrange the hierarchy
		for (var i = 0; i < this.childNodes; i++)
			this.childNodes[i].setHierarchy();

		this.setColor();
	},
	
    // picks up a color based on the hierarchy level
	setColor: function()
	{
		var hierarchyLevel = this.hierarchy % 6;
		switch (hierarchyLevel)
		{
		case 0:
			this.setBrush("maroon");
			break;
		case 1:
		    this.setBrush("maroon");
			break;
		case 2:
		    this.setBrush("maroon");
			break;
		case 3:
		    this.setBrush("maroon");
			break;
		case 4:
		    this.setBrush("maroon");
			break;
		case 5:
		    this.setBrush("maroon");
			break;
		case 6:
		    this.setBrush("maroon");
			break;
		default:
		    this.setBrush("maroon");
			break;
		}
	},

    // adds a new employee
	addChild: function(child)
	{
        //checks if the employee already exists
		var isHere = false;
		for (var i=0 ; i<this.childNodes.length;i++)
		{
			if(this.childNodes[i].id == child.id)
				isHere = true;
		}
		if (!isHere)
			this.childNodes.push(child);
	},

    // removes the specified child from the collection of nodes
	removeChild: function(child)
	{
		var index = this.childNodes.indexOf(child);
		if(index > -1)
			this.childNodes.splice(index,1);
	},

    // rearranges the hierarchy
	resetHierarchy: function()
	{
		this.setHierarchy();
		for (var i=0 ; i<this.childNodes.length;i++)
		{
			this.childNodes[i].setBoss(this);
			this.childNodes[i].resetHierarchy();}
	},

    // checks if the specified node is a child node
	isChild: function(child)
	{
		for (var i=0;i<this.childNodes.length; i++)
		{
			if (child === this.childNodes[i])
				return true;
		}
		return false;
	}
};

var diagram = null;
var tree = null;

$(document).ready(function () {
    AbstractionLayer.registerClass(OrgChartNode, "OrgChartNode", MindFusion.Diagramming.TableNode);

    // we apply the tree layout to arrange the diagram
    tree = new MindFusion.Graphs.TreeLayout();
    // customize the tree layout
    tree.direction = MindFusion.Graphs.LayoutDirection.TopToBottom;
    tree.linkType = MindFusion.Graphs.TreeLayoutLinkType.Cascading;

    // create the diagram object and apply customization
    $("#diagram")[0].oncontextmenu = function () { return false; }
    diagram = Diagram.create($("#diagram")[0]);
        var defaultStyle = new Style();
        defaultStyle.setTextColor("Yellow");
        defaultStyle.setFontSize(7);
        defaultStyle.setFontStyle("Bold");
        diagram.setStyle(defaultStyle);

    diagram.setBehavior(Behavior.Custom);
    diagram.setShowGrid(false);
    diagram.setAllowSelfLoops(false);
    diagram.setAutoScroll(true);
    diagram.setLinkShape(LinkShape.Cascading);
      diagram.setBackBrush('red');

    //var pfc = new PrivateFontCollection();
    //pfc.AddFontFile("C:\Users\Akshay\Downloads\Terafonts\TERAFONT-PARTH.TTF");
    //myLabel.Font = new System.Drawing.Font(pfc.Families[0], 20, System.Drawing.FontStyle.Regular);

    //add an EventListener for clicked events
    diagram.addEventListener(Event.clicked, function (diagram, eventArgs) {
        // check which mouse button was clicked
        var button = eventArgs.getMouseButton();
        var position = eventArgs.getMousePosition();

        // click with the right mouse button creates a node
        if (button === 2) {
            var node = new OrgChartNode(diagram, undefined);
            node.setBounds(new Rect(position.x, position.y, 5, 5));
            node.resize();

            // adds the node to the diagram items
            diagram.addItem(node);

            // rearrange the diagram
            diagram.arrangeAnimated(tree);
        }
    });

    diagram.addEventListener(Event.animatedLayoutCompleted, function (diagram, eventArgs) {
        // resize the diagram to fit all items
        diagram.resizeToFitItems();
    });

    diagram.addEventListener(Event.nodeClicked, onNodeClicked);

    // add an EventListener for nodeSelected events
    diagram.addEventListener(Event.nodeSelected, function (diagram, eventArgs) {
        // make sure the clicked node appears above any other nodes at the same location
        eventArgs.getNode().setZIndex(0);
    });

    // raised when the user deselcts a node
    diagram.addEventListener(Event.nodeDeselected, function (diagram, eventArgs) {
        //reset the z-index of the node
        eventArgs.getNode().setZIndex(0);
    });

    //add an eventListener for the linkCreated event
    diagram.addEventListener(Event.linkCreated, onLinkCreated);

    //add an eventListener for the cellTextEdited event
    diagram.addEventListener(Event.cellTextEdited, function (diagram, cellArgs) {
        cellArgs.getCell().onEdited(diagram, cellArgs);
    });

    // add an EventLitener for the linkCreated event
    diagram.addEventListener(Event.linkCreated, function (diagram, eventArgs) {
        diagram.arrangeAnimated(tree);
    });

    // add an eventListener for the nodeDeleted event
    diagram.addEventListener(Event.nodeDeleted, function (diagram, eventArgs) {
        diagram.arrangeAnimated(tree);
    })

    // add an eventListener for the linkDeleted event 
    diagram.addEventListener(Event.linkDeleted, function (diagram, eventArgs) {
        // gets the destinationNode for the link that is deleted
        var destNode = eventArgs.getLink().destination;

        // remove the destinationNode as a subordinate to the boss of the destination node  
        destNode.boss.removeChild(destNode);

        // destinationNode now has no boss
        destNode.boss = undefined;

        // reset the hierarchy for the destination node
        destNode.resetHierarchy();
        destNode.getCell(0, 0).setText("");
        diagram.arrangeAnimated(tree);
    });

    // handles nodeModified events
    diagram.addEventListener(Event.nodeModified, function (diagram, eventArgs) {
        // if the modified node does not have a boss
        if (!(eventArgs.getNode().boss)) {
            // get the mouse position
            var mousePoint = eventArgs.getMousePosition();
            // and all nodes at this position
            var nodes = diagram.getNodesAt(mousePoint);

            for (var i = 0; i < nodes.length; i++) {
                // if the node is not the one being modified and is not child of the one being modified
                if (nodes[i] !== eventArgs.getNode() && !eventArgs.getNode().isChild(nodes[i])) {
                    // set nodes[i] as boss for the modified node
                    eventArgs.getNode().setBoss(nodes[i]);

                    // create a diagramLink to show the hierarchy
                    var l = diagram.getFactory().createDiagramLink(nodes[i], eventArgs.getNode());

                    // apply link design
                    styleLink(l);

                    // reset the node hierarchy
                    eventArgs.getNode().resetHierarchy();

                    // rearrange the diagram
                    diagram.arrangeAnimated(tree);

                    // resize the diagram to fit all items
                    diagram.resizeToFitItems();
                    break;
                }
            }
        }
    })

    //add an EventListener for inPlaceEdit events 
    diagram.addEventListener(Event.enterInplaceEditMode, function (diagram, eventArgs) {
        var cell = eventArgs.getItem().getCellKind();
        //identify the cell that is editied
        switch (cell) {
            case 0:
                // the image
                //eventArgs.getControl().setAttribute("placeholder", "Image URL:");
                break;

            case 1:
                // the name
                eventArgs.getControl().setAttribute("placeholder", "Name:");
                break;

            case 2:
                // the title
                //eventArgs.getControl().setAttribute("placeholder", "Title:");
                break;

            default:
                // comments
                //eventArgs.getControl().setAttribute("placeholder", "Comments:");
                break;
        }
    });

    // create some sample nodes e.g. employees

    

    var kanji = new OrgChartNode(diagram);
    kanji.setBounds(new Rect(25, 15, 10, 10));
    //kanji.setTitle("CEO");
    kanji.setFullName("SFGÒ");
    //kanji.setImageLocation("ceo.png");
    //kanji.setComment("The CEO of this great company");
    kanji.resize();
    diagram.addItem(kanji);

    //    var pfc = new PrivateFontCollection();
    //    pfc.AddFontFile("C:\Users\Akshay\Downloads\Terafonts\TERAFONT-PARTH.TTF");
    //    kanji.Font = new System.Drawing.Font(pfc.Families[0], 20, System.Drawing.FontStyle.Regular);

    var surji = new OrgChartNode(diagram, kanji);
    surji.setBounds(new Rect(25, 55, 10, 10));
    surji.setBoss(kanji);
    //surji.setTitle("CTO");
    surji.setFullName(";]ZÒ");
    //surji.setImageLocation("cto.png");
    //surji.setComment("A great person!");
    surji.resize();
    diagram.addItem(surji);

    var bhanji = new OrgChartNode(diagram, kanji);
    bhanji.setBounds(new Rect(95, 55, 10, 10));

    //bhanji.setTitle("HR");
    bhanji.setFullName("EF^FÒ");
    //bhanji.setImageLocation("hr.png");
    //bhanji.setComment("Human Relations Manager");
    bhanji.resize();
    diagram.addItem(bhanji);


    var ladha = new OrgChartNode(diagram, bhanji);
    ladha.setBounds(new Rect(95, 95, 10, 10));
    //ladha.setTitle("PR");
    ladha.setFullName(",WFvJLZAF>");
    //ladha.setImageLocation("pr.png");
    ladha.resize();
    diagram.addItem(ladha);

    var samji = new OrgChartNode(diagram, ladha);
    samji.setBounds(new Rect(95, 135, 10, 15));
    //samji.setTitle("Media");
    samji.setFullName(";FDÒvDFGAF>");
    //samji.setImageLocation("ad.png");
    samji.resize();
    diagram.addItem(samji);

    var ramji = new OrgChartNode(diagram, samji);
    ramji.setBounds(new Rect(95, 175, 10, 15));
    //ramji.setTitle("Media");
    ramji.setBoss(samji);
    ramji.setFullName("ZFDÒvC\\;FA[G");
    //ramji.setImageLocation("ad.png");
    ramji.resize();
    diagram.addItem(ramji);

    var kantaben = new OrgChartNode(diagram, samji);
    kantaben.setBounds(new Rect(95, 175, 10, 15));
    //naran.setTitle("Media");
    kantaben.setFullName("SF\TFA[G");
    //naran.setImageLocation("ad.png");
    kantaben.resize();
    diagram.addItem(kantaben);

    var naran = new OrgChartNode(diagram, samji);
    naran.setBounds(new Rect(95, 175, 10, 15));
    //naran.setTitle("Media");
    naran.setFullName("GFZF^FvhJ[ZA[G");
    //naran.setImageLocation("ad.png");
    naran.resize();
    diagram.addItem(naran);

    var shantaben = new OrgChartNode(diagram, samji);
    shantaben.setBounds(new Rect(95, 175, 10, 15));
    //naran.setTitle("Media");
    shantaben.setFullName("XF\TFA[G");
    //naran.setImageLocation("ad.png");
    shantaben.resize();
    diagram.addItem(shantaben);

    var Hiravatiben = new OrgChartNode(diagram, samji);
    Hiravatiben.setBounds(new Rect(95, 175, 10, 15));
    //naran.setTitle("Media");
    Hiravatiben.setFullName("CLZFJlTA[G");
    //naran.setImageLocation("ad.png");
    Hiravatiben.resize();
    diagram.addItem(Hiravatiben);

    var Kasturben = new OrgChartNode(diagram, samji);
    Kasturben.setBounds(new Rect(95, 175, 10, 15));
    //naran.setTitle("Media");
    Kasturben.setFullName("S:T]ZA[G");
    //naran.setImageLocation("ad.png");
    Kasturben.resize();
    diagram.addItem(Kasturben);

    var bhanu = new OrgChartNode(diagram, ramji);
    bhanu.setBounds(new Rect(95, 215, 10, 15));
    //bhanu.setTitle("Media");
    bhanu.setFullName("EFG]");
    bhanu.setImageLocation("ad.png");
    bhanu.resize();
    diagram.addItem(bhanu);

    var Pravina = new OrgChartNode(diagram, naran);
    Pravina.setBounds(new Rect(95, 215, 10, 15));
    //bhanu.setTitle("Media");
    Pravina.setFullName("5|lJ6F");
    Pravina.setImageLocation("ad.png");
    Pravina.resize();
    diagram.addItem(Pravina);
    var Ramesh = new OrgChartNode(diagram, naran);
    Ramesh.setBounds(new Rect(95, 215, 10, 15));
    //Ramesh.setTitle("Media");
    Ramesh.setFullName("ZD[;");
    Ramesh.setImageLocation("ad.png");
    Ramesh.resize();
    diagram.addItem(Ramesh);
    var Daksha = new OrgChartNode(diagram, ramji);
    Daksha.setBounds(new Rect(95, 215, 10, 15));
    //bhanu.setTitle("Media");
    Daksha.setFullName("N1FF");
    Daksha.setImageLocation("ad.png");
    Daksha.resize();
    diagram.addItem(Daksha);
    var Kishan = new OrgChartNode(diagram, naran);
    Kishan.setBounds(new Rect(95, 215, 10, 15));
    //Kishan.setTitle("Media");
    Kishan.setFullName("SL;Gv;LwWL");
    Kishan.setImageLocation("ad.png");
    Kishan.resize();
    diagram.addItem(Kishan);
    var ritesh = new OrgChartNode(diagram, ramji);
    ritesh.setBounds(new Rect(95, 215, 10, 15));
    //ritesh.setTitle("Media");
    ritesh.setBoss(ramji);
    ritesh.setFullName("ZLT[QFvR[TGF");
    //ritesh.setImageLocation("ad.png");
    ritesh.resize();
    diagram.addItem(ritesh);
    var Sita = new OrgChartNode(diagram, ramji);
    Sita.setBounds(new Rect(95, 215, 10, 15));
    //Sita.setTitle("Media");
    Sita.setFullName(";LTF");
    Sita.setImageLocation("ad.png");
    Sita.resize();
    diagram.addItem(Sita);
    var Nilesh = new OrgChartNode(diagram, ramji);
    Nilesh.setBounds(new Rect(95, 215, 10, 15));
    //Nilesh.setTitle("Media");
    Nilesh.setBoss(ramji);
    Nilesh.setFullName("GL,[;vEFlJSF");
    //Nilesh.setImageLocation("ad.png");
    Nilesh.resize();
    diagram.addItem(Nilesh);
    var Anusuya = new OrgChartNode(diagram, naran);
    Anusuya.setBounds(new Rect(95, 215, 10, 15));
    //bhanu.setTitle("Media");
    Anusuya.setFullName("VG];]IF");
    Anusuya.setImageLocation("ad.png");
    Anusuya.resize();
    diagram.addItem(Anusuya);
    var Aarav = new OrgChartNode(diagram, Nilesh);
    Aarav.setBounds(new Rect(95, 215, 10, 15));
    //Kishan.setTitle("Media");
    Aarav.setFullName("VFZJ");
    //Aarvi.setImageLocation("ad.png");
    Aarav.resize();
    diagram.addItem(Aarav);


    var Krishna = new OrgChartNode(diagram, Kishan);
    Krishna.setBounds(new Rect(95, 215, 10, 15));
    //Kishan.setTitle("Media");
    Krishna.setFullName("lS|QGF");
    Krishna.setImageLocation("ad.png");
    Krishna.resize();
    diagram.addItem(Krishna);

    var Aarvi = new OrgChartNode(diagram, ritesh);
    Aarvi.setBounds(new Rect(95, 215, 10, 15));
    //Kishan.setTitle("Media");
    Aarvi.setFullName("VFZlJ");
    //Aarvi.setImageLocation("ad.png");
    Aarvi.resize();
    diagram.addItem(Aarvi);


    // link the sample nodes and apply the link design to each one
    var link2 = diagram.getFactory().createDiagramLink(kanji, surji);
    styleLink(link2);
    var link3 = diagram.getFactory().createDiagramLink(kanji, bhanji);
    styleLink(link3);
    var link4 = diagram.getFactory().createDiagramLink(bhanji, ladha);
    styleLink(link4);
    var link5 = diagram.getFactory().createDiagramLink(ladha, samji);
    styleLink(link5);
    var link6 = diagram.getFactory().createDiagramLink(samji, ramji);
    styleLink(link6);
    var link7 = diagram.getFactory().createDiagramLink(samji, naran);
    styleLink(link7);
    var link8 = diagram.getFactory().createDiagramLink(ramji, ritesh);
    styleLink(link8);
    var link9 = diagram.getFactory().createDiagramLink(ramji, Nilesh);
    styleLink(link9);
    var link10 = diagram.getFactory().createDiagramLink(naran, Ramesh);
    styleLink(link10);
    var link11 = diagram.getFactory().createDiagramLink(naran, Kishan);
    styleLink(link11);
    var link12 = diagram.getFactory().createDiagramLink(ritesh, Aarvi);
    styleLink(link12);
    var link13 = diagram.getFactory().createDiagramLink(samji, kantaben);
    styleLink(link13);
    var link14 = diagram.getFactory().createDiagramLink(samji, shantaben);
    styleLink(link14);
    var link15 = diagram.getFactory().createDiagramLink(samji, Hiravatiben);
    styleLink(link15);
    var link16 = diagram.getFactory().createDiagramLink(samji, Kasturben);
    styleLink(link16);
    var link17 = diagram.getFactory().createDiagramLink(Kishan, Krishna);
    styleLink(link17);
    var link18 = diagram.getFactory().createDiagramLink(Nilesh, Aarav);
    styleLink(link18);
    var link19 = diagram.getFactory().createDiagramLink(ramji, bhanu);
    styleLink(link19);
    var link20 = diagram.getFactory().createDiagramLink(naran, Pravina);
    styleLink(link20);
    var link21 = diagram.getFactory().createDiagramLink(ramji, Daksha);
    styleLink(link21);
    var link22 = diagram.getFactory().createDiagramLink(ramji, Sita);
    styleLink(link22);
    var link23 = diagram.getFactory().createDiagramLink(naran, Anusuya);
    styleLink(link23);
    // arrange the diagram with the TreeLayout
    diagram.arrangeAnimated(tree);
    // disable built-in inplace editing (here it's triggered by different events than double-click)
    diagram.setAllowInplaceEdit(false);
    // set diagram behavior
    diagram.setBehavior(Behavior.Modify);
});

// called when the user edits a table
function editNode(diagram, eventArgs)
{
	var cellEditor = eventArgs.getNode().cellFromPoint(eventArgs.getMousePosition());

    // the table node to edit
	var tableNode = eventArgs.getNode();
	var edit = false;

    // cells that cannot be edited have no cellEditor assigned
	if (cellEditor.cell != undefined)
		if(cellEditor.cell.editable == true)
			edit = true;

	// if the cell can be edited
	if (edit)
	{
		diagram.beginEdit(eventArgs.getNode(),eventArgs.getMousePosition());
		$("diagram_inplaceInput").attr("placeholder", "url");
		cellEditor.cell.onEdited = function (diagram, tableCell)
		{
			if (edit)
			{
				if (cellEditor.cell.image)
				{
					// read and assign the URL of the new image
					if (tableCell.getNewText() != undefined &&
						tableCell.getNewText() != "" &&
						tableCell.getNewText != "undefined")
					{
						tableNode.setImageLocation(tableCell.getNewText());
						cellEditor.cell.text.height = 0;
						cellEditor.cell.text.text = "";
					}
				}
				//resize the table cell to fit the new image
				else if (cellEditor.row < 3)
				{
					var oldImageWidth = tableNode.getColumn(0).width;
					tableNode.resizeToFitText(true,true);
					tableNode.getColumn(0).width = oldImageWidth;
					tableNode.bounds.width += oldImageWidth;					
				}

				else if (cellEditor.row === 3)
				{
					var oldHeight = tableNode.bounds.height;
					var oldImageWidth = tableNode.getColumn(0).width;
					tableNode.resizeToFitText(true,true);
					var newHeight = tableNode.bounds.height - oldHeight;
					tableNode.getCell(0, 0).imagePadding.top += newHeight / 1;
					tableNode.getCell(0, 0).imagePadding.bottom += newHeight / 1;
					tableNode.getColumn(0).width = oldImageWidth;
					tableNode.bounds.width += oldImageWidth;
				}
				
			    if (cellEditor.row == 0 && cellEditor.column == 2)
                 tableNode.fullName = tableCell.newText;
                if (cellEditor.row == 1 && cellEditor.column == 2)
                 tableNode.title = tableCell.newText;
                if (cellEditor.row == 3 && cellEditor.column == 1)
                 tableNode.comments = tableCell.newText;
			 
				diagram.arrangeAnimated(tree);
			}
		}
        // checks to see if the edited object is an image
		tableNode.getCellKind = function()
		{
			if (cellEditor.cell.image)
				return 0;
			if (cellEditor.row == 0)
				return 1;
			if (cellEditor.row == 1)
				return 2;
			if (cellEditor.row == 3)
				return 3;
			return 4;
		}
	}
}

// raised when the user creates a link
function onLinkCreated(diagram, eventArgs)
{
	if (eventArgs.getLink().permission === true)
		styleLink(eventArgs.getLink());
}

function styleLink(link)
{
	link.setPen("black");
	link.setStrokeThickness(3.0);
	link.setHeadShape();
}

// raised when the user creates a node
function createNode(diagram, eventArgs)
{
	var parent = eventArgs.getNode();
	var node = new OrgChartNode(diagram, parent);
	node.setBounds(parent.getBounds());

	diagram.addItem(node);
	node.setComment("(comment)");
	node.resize();

    // create a link that connects the new node
	var link = diagram.getFactory().createDiagramLink(eventArgs.getNode(),node);
	link.permission = true;

    // apply the link design
	styleLink(link);

    // rearrange the diagram
	diagram.arrangeAnimated(tree);

    // resize the diagram to fit all nodes
	diagram.resizeToFitItems();
}

// raised when the user clicks on a node
function onNodeClicked(diagram, eventArgs)
{
	// checks if the user has clicked with the left mouse button (0)
	var button = eventArgs.getMouseButton();
	if (button === 0)
		editNode(diagram, eventArgs);

    //click with the right mouse button creates a node
	else if (button === 2)
		createNode(diagram, eventArgs);
}